/******************************************************************
 *
 *  SpiderX25 Protocol Streams Component
 *
 *  Copyright 1991  Spider Systems Limited
 * Copyright 31 Jan 1995 Sun Microsystems, Inc. All Rights Reserved
 *
 *  LL_CONTROL.H
 *
 *    LAPB Top-level STREAMS interface.
 *
 ******************************************************************/

#ifndef _NETDLC_LL_CONTROL_H
#define _NETDLC_LL_CONTROL_H

/*
 *	ll_control.h
 *	 /projects/common/PBRAIN/SCCS/pbrainF/dev/src/include/sys/snet/0/s.ll_control.h
 *	@(#)ll_control.h	1.28
 *
 *	Last delta created	18:35:19 10/7/92
 *	This file extracted	18:58:08 11/24/92
 *
 */

/* IOCTL commands */
#define L_SETPPA    ('L'<<8 | 1) /* Set PPA (use ll_snioc)                   */
#define L_GETPPA    ('L'<<8 | 2) /* Get PPA (use ll_snioc)                   */
#define L_SETTUNE   ('L'<<8 | 3) /* Set tuning parameters (use ll_tnioc)     */
#define L_GETTUNE   ('L'<<8 | 4) /* Get tuning parameters (use ll_tnioc)     */
#define L_GETSTATS  ('L'<<8 | 5) /* Get statistics counts (use ll_stioc)     */
#define L_ZEROSTATS ('L'<<8 | 6) /* Zero statistics       (use ll_hdioc)     */
#define L_TRACEON   ('L'<<8 | 7) /* Set message tracing on.                  */
#define L_TRACEOFF  ('L'<<8 | 8) /* Set message tracing off.                 */
#define L_GETGSTATS ('L'<<8 | 10) /* Get global stats counts (use ll_gstioc) */
#define L_ASYNC_NOTIF ('L'<<8 | 11) /* User wants async notification of XID
                                     * and TEST timeouts. */
#define L_DISAB_NOTIF ('L'<<8 | 12) /* Disable async notification. */
#define L_GET_LINKID  ('L'<<8 | 13) /* Get linkid of stream */

/* New ones to allow start and stop of an individual link */
#define L_STARTLINK  ('L'<<8 | 20)	/* set link to running state */
#define L_STOPLINK  ('L'<<8 | 21)	/* set link to disconnected state */
#define L_RESETLINK  ('L'<<8 | 22)	/* set link to disconnected state */

/* Spider's old IOCTL names */
#define L_SETSNID   ('L'<<8 | 1) /* Set subnetwork identifier (use ll_snioc) */
#define L_GETSNID   ('L'<<8 | 2) /* Get subnetwork identifier (use ll_snioc) */

/* Values for 'lli_type' (with names of corresponding structures) */
#define LI_PLAIN	0x01        /* Indicates 'struct ll_hdioc'  */
#define LI_SPPA		0x02        /* Indicates 'struct ll_snioc'  */
#define LI_SNID		0x02        /* Indicates 'struct ll_snioc'  */
#define LI_STATS	0x04        /* Indicates 'struct ll_stioc'  */
#define LI_GSTATS	0x16        /* Indicates 'struct ll_gstioc' */

#define LI_LAPBTUNE	0x13        /* Indicates 'struct lapb_tnioc'*/
#define LI_LLC2TUNE	0x23        /* Indicates 'struct llc2_tnioc'*/

/* LAPB tuning structure */
typedef struct lapbtune {
    uint16  N2;		    /* Maximum number of retries		*/
    uint16  T1;		    /* Acknowledgement time	(unit 0.1 sec)  */
    uint16  Tpf;            /* P/F cycle retry time	(unit 0.1 sec)  */
    uint16  Trej;           /* Reject retry time	(unit 0.1 sec)  */
    uint16  Tbusy;          /* Remote busy check time   (unit 0.1 sec)  */
    uint16  Tidle;          /* Idle P/F cycle time	(unit 0.1 sec)  */
    uint16  ack_delay;      /* RR delay time		(unit 0.1 sec)  */
    uint16  notack_max;     /* Maximum number of unack'ed Rx I-frames   */
    uint16  tx_window;      /* Transmit window size			*/
    uint16  tx_probe;       /* P-bit position before end of Tx window   */
    uint16  max_I_len;      /* Maximum I-frame length			*/
    uint16  llconform;      /* LAPB conformance				*/
} lapbtune_t;

/* Values for llconform field in lapbtune_t  */

#define	IGN_UA_ERROR       1	/* UA frames ignored in ERROR state	*/
#define	FRMR_FRMR_ERROR    2	/* FRMR is retransmitted in ERROR state	*/
#define	FRMR_INVRSP_ERROR  4	/* Invalid response frames are FRMR'ed	*/
#define	SFRAME_PBIT        8	/* S frame commands must have P-bit set	*/
#define	NO_DM_ADM         16	/* Dont send DM on entry to ADM state   */
#define	ISO8882		  32	/* Variant for ISO8882 conformance	*/
#define	INITIAL_DOWN	  64	/* SAND Link is initially DOWN 		*/

/* Special temporary private flag used only internally */
#define LINK_OFF	0x80	/* LINKSTOP has been done on this */

/* LLC2 tuning structure */
typedef struct llc2tune {
    uint16  N2;		    /* Maximum number of retries		*/
    uint16  T1;		    /* Acknowledgement time	(unit 0.1 sec)  */
    uint16  Tpf;            /* P/F cycle retry time	(unit 0.1 sec)  */
    uint16  Trej;           /* Reject retry time	(unit 0.1 sec)  */
    uint16  Tbusy;          /* Remote busy check time   (unit 0.1 sec)  */
    uint16  Tidle;          /* Idle P/F cycle time	(unit 0.1 sec)  */
    uint16  ack_delay;      /* RR delay time		(unit 0.1 sec)  */
    uint16  notack_max;     /* Maximum number of unack'ed Rx I-frames   */
    uint16  tx_window;      /* Transmit window (if no XID received)	*/
    uint16  tx_probe;       /* P-bit position before end of Tx window   */
    uint16  max_I_len;      /* Maximum I-frame length			*/
    uint16  xid_window;     /* XID window size (receive window)		*/
    uint16  xid_Ndup;       /* Duplicate MAC XID count  (0 => no test)  */
    uint16  xid_Tdup;       /* Duplicate MAC XID time   (unit 0.1 sec)  */
} llc2tune_t;

/* Header alone (for decoding and L_ZEROSTATS commands) */
struct ll_hdioc {
    uint8           lli_type;      /* Table type = LI_PLAIN	*/
    uint8           lli_spare[3];  /*   (for alignment)		*/
    uint32          lli_ppa;       /* PPA/ Subnetwork ID char   */
};

/* Ioctl block for L_SETPPA and L_GETPPA commands */
struct ll_snioc {
    uint8           lli_type;      /* Table type = LI_SPPA		*/
    uint8           lli_spare[3];  /*   (for alignment)			*/
    uint32          lli_ppa;       /* PPA/ Subnetwork ID character	*/
    uint32          lli_index;     /* Link index			*/
};


/* Ioctl block for LAPB L_SETTUNE and L_GETTUNE commands */
struct lapb_tnioc {
    uint8           lli_type;      /* Table type = LI_LAPBTUNE		  */
    uint8           lli_spare[3];  /*   (for alignment)			  */
    uint32          lli_ppa;       /* PPA/ Subnetwork ID char (0xff => 'all') */

    lapbtune_t      lapb_tune;     /* Table of tuning values		  */
};

/* Ioctl block for LLC2 L_SETTUNE and L_GETTUNE commands */
struct llc2_tnioc {
    uint8           lli_type;      /* Table type = LI_LLC2TUNE		  */
    uint8           lli_spare[3];  /*   (for alignment)			  */
    uint32          lli_ppa;       /* PPA/ Subnetwork ID char (0xff => 'all') */

    llc2tune_t      llc2_tune;     /* Table of tuning values		  */
};


/* Statistics table definitions 	*/
/* Both LAPB and LLC2 			*/
#define tx_ign		    0	/* no. ignored + not sent	*/
#define rx_badlen	    1	/* bad length frames received	*/
#define rx_unknown 	    2	/* unknown frames received	*/
#define t1_exp		    3	/* no. of T1 timeouts		*/
#define t4_exp		    4	/* no. of T4 timeouts		*/
#define t4_n2_exp	    5	/* T4 timouts after N2 times	*/

#define RR_rx_cmd           6	/* RR = Receive Ready		*/
#define RR_rx_rsp           7	/* tx = transmitted		*/
#define RR_tx_cmd           8	/* rx = received		*/
#define RR_tx_rsp           9	/* cmd/rsp = command/response	*/
#define RR_tx_cmd_p        10	/* p = p-bit set      		*/

#define RNR_rx_cmd         11	/* RNR = Receive Not Ready	*/
#define RNR_rx_rsp         12
#define RNR_tx_cmd         13
#define RNR_tx_rsp         14
#define RNR_tx_cmd_p       15

#define REJ_rx_cmd         16	/* REJ = Reject			*/
#define REJ_rx_rsp         17
#define REJ_tx_cmd         18
#define REJ_tx_rsp         19 
#define REJ_tx_cmd_p       20

#define SABME_rx_cmd       21	/* SABME = Set Asynchronous 	*/
#define SABME_tx_cmd       22	/*	 Balanced Mode Extended	*/

#define DISC_rx_cmd        23	/* DISC = Disconnect		*/
#define DISC_tx_cmd        24

#define UA_rx_rsp          25	/* UA = Unnumbered 		*/
#define UA_tx_rsp          26	/*	Acknowledement		*/

#define DM_rx_rsp          27	/*				*/
#define DM_tx_rsp          28

#define I_rx_cmd           29	/* I = Information		*/
#define I_tx_cmd           30

#define FRMR_rx_rsp        31	/* FRMR = Frame Reject		*/
#define FRMR_tx_rsp        32 

#define tx_rtr		   33	/* no. of retransmitted frames 	*/
#define rx_bad		   34	/* erroneous frames received   	*/
#define rx_dud		   35	/* received and discarded     	*/
#define rx_ign		   36	/* received and ignored        	*/

/* LAPB only */
#define SABM_rx_cmd        37   /* SABM = Set Asynchronous	*/
#define SABM_tx_cmd        38 	/*	  Balanced Mode		*/
#define SARM_rx_cmd        39 	/* SARM = Set Asynchronous	*/
#define SARM_tx_cmd        40	/*	  Response Mode		*/

#define lapbstatmax        41

/* LLC2 only */
/* values mutually exclusive with LAPB */

#define I_rx_rsp	   37	
#define I_tx_rsp           38

#define UI_rx_cmd          39	
#define UI_tx_cmd          40

#define XID_rx_cmd         41
#define XID_rx_rsp         42
#define XID_tx_cmd         43
#define XID_tx_rsp         44

#define TEST_rx_cmd        45
#define TEST_rx_rsp        46
#define TEST_tx_cmd        47 
#define TEST_tx_rsp        48

#define llc2statmax        49

/* Global L2 statistics */
#define frames_tx	    0	/* frames transmitted		*/
#define frames_rx	    1	/* frames received		*/
#define sabm_tx		    2	/* for LAPB  			*/	
#define sabm_rx             3   /* for LAPB                     */
#define sabme_tx	    2   /* for LLC-2 			*/
#define sabme_rx            3   /* for LLC-2                    */
#define bytes_tx	    4	/* data bytes transmitted	*/
#define bytes_rx	    5	/* data bytes received		*/
#define globstatmax	    6	/* size of global stats array	*/


typedef struct llc2_stats {
	uint32	llc2monarray[llc2statmax]; /* array of LLC2 stats */
} llc2stats_t;

typedef struct lapb_stats {
	uint32	lapbmonarray[lapbstatmax]; /* array of LAPB stats */
} lapbstats_t;

/* Ioctl block for L_GETSTATS command (per-link) */
/*
 * SMcK Added state 20th January 1994 for linkstate command. The
 * padding was changed so that the structure alignment remains the
 * same.
*/
struct llc2_stioc {
    uint8           lli_type;	   /* Table type = LI_STATS		*/
    uint8	    state;      /* connection state			*/
    uint16          lli_spare;  /*   (for alignment)			*/
    uint32          lli_ppa;       /* PPA/ Subnetwork ID character	*/
    llc2stats_t     lli_stats;     /* Table of stats values		*/
};

struct lapb_stioc {
    uint8           lli_type;   /* Table type = LI_STATS		*/
    uint8	    state;      /* connection state			*/
    uint16          lli_spare;  /*   (for alignment)			*/
    uint32          lli_ppa;    /* PPA/ Subnetwork ID character		*/
    lapbstats_t     lli_stats;  /* Table of stats values		*/
};

/* Ioctl block for L_GETGSTATS command */
struct llc2_gstioc {
    uint8           lli_type;	   /* Table type = LI_GSTATS		*/
    uint8           lli_spare[3];  /*   (for alignment)			*/
    uint32	    llc2gstats[globstatmax];
				   /* global statistics table           */
};

struct lapb_gstioc {
    uint8           lli_type;      /* Table type = LI_GSTATS		*/
    uint8           lli_spare[3];  /*   (for alignment)			*/
    uint32	    lapbgstats[globstatmax];
				   /* global statistics table           */
};

/* Union of ioctl blocks */
typedef union lli_union {
    struct ll_hdioc	ll_hd;      /* Parameter-less command       	*/
    struct ll_snioc	ll_sn;      /* Set/get subnetwork identifier    */
    struct lapb_tnioc   lapb_tn;    /* Set/get LAPB tuning          	*/
    struct llc2_tnioc   llc2_tn;    /* Set/get LLC2 tuning          	*/
    struct llc2_stioc   llc2_st;    /* Get llc2 per-link statistics  	*/
    struct lapb_stioc   lapb_st;    /* Get lapb per-link statistics   	*/
    struct llc2_gstioc  llc2_gst;   /* Get llc2 global statistics      	*/
    struct lapb_gstioc  lapb_gst;   /* Get lapb global statistics     	*/
} lliun_t;

/*
 * Define 'lli_snid' for compatibility with Spider nomenclature
 * 'subnetwork ID'.
 */
#define lli_snid	lli_ppa

#endif /* _NETDLC_LL_CONTROL_H */
