/******************************************************************
 *
 *  SpiderX25 - LAPB STREAMS Multiplexer
 *
 *  Copyright 1991 Spider Systems Limited
 *
 *  LAPB.H
 *
 *    Private header file.
 *
 ******************************************************************/

#ifndef _NETX25_LAPB_H
#define _NETX25_LAPB_H

/*
 *	 /usr/projects.old/tcp/PBRAIN/SCCS/pbrainF/dev/sys/lapb/0/s.lapb.h
 *	@(#)lapb.h	1.27
 *
 *	Last delta created	17:34:00 4/30/92
 *	This file extracted	11:25:10 5/14/92
 *
 */

#ident "@(#)lapb.h	1.27 (Spider) 5/14/92"

/* Permanent definitions */
#define A_ADDRESS        3
#define B_ADDRESS        1
#define P_BIT         0x10

/* States */
#define OFF		 0
#define START		 1
#define D_CONN		 2
#define ADM		 3
#define ARM		 4
#define POLLING		 5
#define PRESETUP         6
#define SETUP		 7
#define RESET		 8
#ifdef	ERROR
#undef	ERROR
#endif
#define ERROR		 9
#define NORMAL		 10

/*  Number of timers */
#define NTIMERS   6

/* Timer IDs (index in timer array) */
#define PTIMER    0
#define ACKTIMER  1
#define REJTIMER  2
#define BUSYTIMER 3
#define RRTIMER   4
#define IDLETIMER 5

/* LAPB link control structure */
typedef struct lapblink {
    mblk_t     *ackq;
    mblk_t     *datq;
    mblk_t     *adqtail;
    mblk_t     *rxqueue;
    mblk_t     *rxqtail;
    queue_t    *downq;
    queue_t    *upq;
    uint32	nethandle;	/* for interface to network layer */
    thead_t	thead;
    x25timer_t	tmtab[NTIMERS];
    lapbtune_t  tunetab;
    lapbstats_t statstab;
    int		rxqcnt;
    int		index;
    int		discard;
    uint16	netcid;
    uint32	snid;
    uint8	upcmd;
    uint8	upstatus;
    uint8	seqmask;
    uint8	extended;
    uint8	myaddr;
    uint8	busyflag;
    uint8	awaitflag;
    uint8	clashflag;
    uint8	pollflag;
    uint8	connected;
    uint8	pflag;
    uint8	wclosed;
    uint8	txstopped;
/* Added fix for bugid 1181238; first in patch 101884-03 - DP 29/12/94 */
#ifndef ORIGINAL
    uint8	rxstopped;
#endif /* ORIGINAL */
/* End of fix for bugid 1181238 */

    uint8	state;
    uint8	reject;

    /* 
     *  causeflag is only used in D_CONN and RESET states to remember
     *  originator of request, and is only changed on entry to these states
     */
    uint8	causeflag;

    /*
     *  the following are the sequence bytes and retrycount which are zeroed by
     *  zero_v_vals, which also empties the queues and clears rem_busy,
     *  akqnum and txqnum
     */
    uint8	vr;
    uint8	vs;
    uint8	ar;
    uint8	as;
/* ISO8882  Changed retry_count from uint8  March 19th */
    short	retry_count;
    uint8	rem_busy;
    /* the following are zeroed by lapb_goto_adm */
    uint8	dataflag;
    uint8	sflag;
    /* the following are evaluated when an appropriate input frame is received */
    uint8	need_fbit;
    uint8	need_ack;
   /*
    *  the values used for retry and timing during the lifetime of the
    *  connection, they are copied from lapb_tune[] by lapb_goto_adm
    *  and thereafter are read only
    */
    uint8	frmr_ctl1;
    uint8	frmr_ctl2;
    uint8	frmr_addr;
    uint8	frmr_reason;

    /*
     *  mode indicates LAPB or LAP procedures.
     */
    uint8	mode;
   /*
    * records whether tracing is active or inactive on this subnetwork
    * and the queue to send trace messages up on.
    */
    uint8	trace_active;
    queue_t    *trace_queue;


#ifndef N_WAN_C
    mblk_t     *WAN_queue;
    mblk_t     *WAN_qtail;
    int         WAN_qcnt;
    uint8       WAN_state;
    uint8       WAN_remsize;
    uint8       WAN_remaddr[12];
#endif

/* ESC 501094 sgypsy: qbufcall ID */
    int         qbufcall_id;


} lapb_t;

#define p_timer     tmtab[PTIMER]
#define ack_timer   tmtab[ACKTIMER]
#define rej_timer   tmtab[REJTIMER]
#define busy_timer  tmtab[BUSYTIMER]
#define idle_timer  tmtab[IDLETIMER]
#define RR_timer    tmtab[RRTIMER]

#define		LAPB_MODE	1
#define		LAP_MODE	2


#ifndef N_WAN_C
/*
    WAN states
*/
#define WAN_IDLE  0
#define WAN_CONN  1
#define WAN_ESTB  2
#define WAN_DISC  3
#endif
 

extern unsigned int lapbgmonarray[globstatmax];


/* per-link stats macro */
#define LAPBMONITOR(i) { \
if(lp->statstab.lapbmonarray[i]<0XFFFFFFFF) \
	lp->statstab.lapbmonarray[i]++; \
}

/* global stats macros */
#define LAPBGMONITOR(i) { \
if(lapbgmonarray[i]<0XFFFFFFFF) \
	lapbgmonarray[i]++; \
}

#define LAPBGMONITORN(i,j) { \
if(lapbgmonarray[i]<(0XFFFFFFFF-j)) \
	lapbgmonarray[i]+=j; \
}

#endif _NETX25_LAPB_H
